# -----------------------------------------------------------
#  Copyright (C) 2008 StatPro Italia s.r.l.
#
#  StatPro Italia
#  Via G. B. Vico 4
#  I-20123 Milano
#  ITALY
#
#  phone: +39 02 96875 1
#  fax:   +39 02 96875 605
#
#  This program is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE. See the license for more details.
# -----------------------------------------------------------
#
'''
This module exists solely for version information so I only have to change it
in one place. Based on the suggestion `here. <http://bit.ly/16LbuJF>`_

:author: Dan Blanchard (dan.blanchard@gmail.com)
'''

__version__ = '0.7.8'
VERSION = tuple(int(x) for x in __version__.split('.'))



# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l.
#
#  StatPro Italia
#  Via G. B. Vico 4
#  I-20123 Milano
#  ITALY
#
#  phone: +39 02 96875 1
#  fax:   +39 02 96875 605
#
#  email: info@riskmap.net
#
#  This program is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE. See the license for more details.
# -----------------------------------------------------------
#
#  Author: Enrico Sirola <enrico.sirola@statpro.com>

"""
drmaa errors
"""

from ctypes import create_string_buffer

from drmaa.const import ERROR_STRING_BUFFER


class DrmaaException(Exception):

    """
    A common ancestor to all DRMAA Error classes.
    """
    pass


class AlreadyActiveSessionException(DrmaaException):
    pass


class AuthorizationException(DrmaaException):
    pass


class ConflictingAttributeValuesException(DrmaaException, AttributeError):
    pass


class DefaultContactStringException(DrmaaException):
    pass


class DeniedByDrmException(DrmaaException):
    pass


class DrmCommunicationException(DrmaaException):
    pass


class DrmsExitException(DrmaaException):
    pass


class DrmsInitException(DrmaaException):
    pass


class ExitTimeoutException(DrmaaException):
    pass


class HoldInconsistentStateException(DrmaaException):
    pass


class IllegalStateException(DrmaaException):
    pass


class InternalException(DrmaaException):
    pass


class InvalidAttributeFormatException(DrmaaException, AttributeError):
    pass


class InvalidContactStringException(DrmaaException):
    pass


class InvalidJobException(DrmaaException):
    pass


class InvalidJobTemplateException(DrmaaException):
    pass


class NoActiveSessionException(DrmaaException):
    pass


class NoDefaultContactStringSelectedException(DrmaaException):
    pass


class ReleaseInconsistentStateException(DrmaaException):
    pass


class ResumeInconsistentStateException(DrmaaException):
    pass


class SuspendInconsistentStateException(DrmaaException):
    pass


class TryLaterException(DrmaaException):
    pass


class UnsupportedAttributeException(DrmaaException, AttributeError):
    pass


class InvalidArgumentException(DrmaaException, AttributeError):
    pass


class InvalidAttributeValueException(DrmaaException, AttributeError):
    pass


class OutOfMemoryException(DrmaaException, MemoryError):
    pass

error_buffer = create_string_buffer(ERROR_STRING_BUFFER)


def error_check(code):
    if code == 0:
        return
    else:
        error_string = "code {0}: {1}".format(code, error_buffer.value.decode())
        try:
            raise _ERRORS[code - 1](error_string)
        except IndexError:
            raise DrmaaException(error_string)

# da vedere: NO_RUSAGE, NO_MORE_ELEMENTS
_ERRORS = [InternalException,
           DrmCommunicationException,
           AuthorizationException,
           InvalidArgumentException,
           NoActiveSessionException,
           OutOfMemoryException,
           InvalidContactStringException,
           DefaultContactStringException,
           NoDefaultContactStringSelectedException,
           DrmsInitException,
           AlreadyActiveSessionException,
           DrmsExitException,
           InvalidAttributeFormatException,
           InvalidAttributeValueException,
           ConflictingAttributeValuesException,
           TryLaterException,
           DeniedByDrmException,
           InvalidJobException,
           ResumeInconsistentStateException,
           SuspendInconsistentStateException,
           HoldInconsistentStateException,
           ReleaseInconsistentStateException,
           ExitTimeoutException,
           Exception,
           StopIteration]



#  tests for drmaa-0.7.8-py35_0 (this is a generated file);
print('===== testing package: drmaa-0.7.8-py35_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
try:
    import drmaa
except RuntimeError as e:
    assert str(e) == "Could not find drmaa library.  Please specify its full path using the environment variable DRMAA_LIBRARY_PATH"
#  --- run_test.py (end) ---

print('===== drmaa-0.7.8-py35_0 OK =====');



# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l.
#
#  StatPro Italia
#  Via G. B. Vico 4
#  I-20123 Milano
#  ITALY
#
#  phone: +39 02 96875 1
#  fax:   +39 02 96875 605
#
#  email: info@riskmap.net
#
#  This program is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE. See the license for more details.
# -----------------------------------------------------------
#
#  Author: Enrico Sirola <enrico.sirola@statpro.com>
#  Author: Dan Blanchard <dan.blanchard@gmail.com>

"""
internal helpers
"""

from __future__ import absolute_import, print_function, unicode_literals

import sys
from collections import namedtuple
from ctypes import (byref, c_uint, create_string_buffer, POINTER, pointer,
                    sizeof)

from drmaa.const import ATTR_BUFFER, ENCODING, NO_MORE_ELEMENTS
from drmaa.errors import error_buffer
from drmaa.wrappers import (drmaa_attr_names_t, drmaa_attr_values_t,
                            drmaa_get_attribute, drmaa_get_attribute_names,
                            drmaa_get_next_attr_name,
                            drmaa_get_next_attr_value,
                            drmaa_get_next_job_id, drmaa_get_vector_attribute,
                            drmaa_get_vector_attribute_names, drmaa_job_ids_t,
                            drmaa_release_attr_names,
                            drmaa_release_attr_values,
                            drmaa_release_job_ids, drmaa_run_bulk_jobs,
                            drmaa_set_attribute, drmaa_set_vector_attribute,
                            drmaa_version, STRING)


# Python 3 compatability help
if sys.version_info < (3, 0):
    bytes = str
    str = unicode


_BUFLEN = ATTR_BUFFER


class BoolConverter(object):

    """Helper class to convert to/from bool attributes."""

    def __init__(self, true=b'y', false=b'n'):
        if isinstance(true, str):
            true = true.encode(ENCODING)
        self.true = true
        if isinstance(false, str):
            false = false.encode(ENCODING)
        self.false = false

    def to_drmaa(self, value):
        if value:
            return self.true
        else:
            return self.false

    def from_drmaa(self, value):
        if value == self.true:
            return True
        else:
            return False


class IntConverter(object):

    """Helper class to convert to/from int attributes."""
    @staticmethod
    def to_drmaa(value):
        return bytes(value)

    @staticmethod
    def from_drmaa(value):
        return int(value)


class SessionStringAttribute(object):

    def __init__(self, drmaa_function):
        self._f = drmaa_function

    def __get__(self, *args):
        buf = create_string_buffer(_BUFLEN)
        c(self._f, buf, sizeof(buf))
        return buf.value.decode()

Version = namedtuple("Version", "major minor")
if sys.version_info < (3, 0):
    Version.__str__ = lambda x: "{0}.{1}".format(x.major,
                                                 x.minor).encode(ENCODING)
else:
    Version.__str__ = lambda x: "{0}.{1}".format(x.major, x.minor)

class SessionVersionAttribute(object):

    """A Version attribute."""

    def __get__(self, *args):
        major = c_uint(10)
        minor = c_uint(10)
        c(drmaa_version, byref(major), byref(minor))
        return Version(major.value, minor.value)


class Attribute(object):

    """
    A DRMAA attribute, to be managed with scalar C DRMAA attribute management
    functions.
    """

    def __init__(self, name, type_converter=None):
        """
        Attribute constructor.

        :Parameters:
         `name` : string
           name of the attribute to be managed, as seen by the underlying C
           DRMAA
         `type_converter`
           a converter to translate attribute values to/from the underlying
           implementation. See BoolConverter for an example.
        """
        if isinstance(name, str):
            name = name.encode(ENCODING)
        self.name = name
        self.converter = type_converter

    def __set__(self, instance, value):
        if self.converter:
            v = self.converter.to_drmaa(value)
        elif isinstance(value, str):
            v = value.encode(ENCODING)
        else:
            v = value
        c(drmaa_set_attribute, instance, self.name, v)

    def __get__(self, instance, _):
        attr_buffer = create_string_buffer(ATTR_BUFFER)
        c(drmaa_get_attribute, instance, self.name, attr_buffer,
          sizeof(attr_buffer))
        if self.converter:
            return self.converter.from_drmaa(attr_buffer.value)
        elif isinstance(attr_buffer.value, bytes):
            return attr_buffer.value.decode()
        else:
            return attr_buffer.value


class VectorAttribute(object):

    """
    A DRMAA attribute representing a list.

    To be managed with vector C DRMAA attribute management functions.
    """

    def __init__(self, name):
        if isinstance(name, str):
            name = name.encode(ENCODING)
        self.name = name

    def __set__(self, instance, value):
        c(drmaa_set_vector_attribute, instance,
          self.name, string_vector(value))

    def __get__(self, instance, _):
        return list(vector_attribute_iterator(instance, self.name))


class DictAttribute(object):

    """
    A DRMAA attribute representing a python dict.

    To be managed with vector C DRMAA attribute management functions.
    """

    def __init__(self, name):
        if isinstance(name, str):
            name = name.encode(ENCODING)
        self.name = name

    def __set__(self, instance, value):
        vector = []
        for k, v in value.items():
            if isinstance(k, bytes):
                k = k.decode(ENCODING)
            if isinstance(v, bytes):
                v = v.decode(ENCODING)
            vector.append("{0}={1}".format(k, v).encode(ENCODING))
        c(drmaa_set_vector_attribute, instance, self.name,
          string_vector(vector))

    def __get__(self, instance, _):
        x = [i.split('=', 1) for i in
             list(vector_attribute_iterator(instance, self.name))]
        return dict(x)


def attributes_iterator(attributes):
    try:
        buf = create_string_buffer(ATTR_BUFFER)
        while drmaa_get_next_attr_value(attributes, buf,
                                        sizeof(buf)) != NO_MORE_ELEMENTS:
            yield buf.value.decode()
    except:
        drmaa_release_attr_values(attributes)
        raise
    else:
        drmaa_release_attr_values(attributes)


def adapt_rusage(rusage):
    """
    Transform a rusage data structure into a dict.

    Due to the value possibly containing a equal sign make sure we
    limit the splits to only the first occurrence.
    """
    rv = dict()
    for attr in attributes_iterator(rusage.contents):

        k, v = attr.split('=',1)
        rv[k] = v
    return rv


def vector_attribute_iterator(jt, attr_name):
    avalues = pointer(POINTER(drmaa_attr_values_t)())
    c(drmaa_get_vector_attribute, jt, attr_name, avalues)
    return attributes_iterator(avalues.contents)


def attribute_names_iterator():
    attrn_p = pointer(POINTER(drmaa_attr_names_t)())
    c(drmaa_get_attribute_names, attrn_p)
    try:
        name = create_string_buffer(_BUFLEN)
        while drmaa_get_next_attr_name(attrn_p.contents, name,
                                       _BUFLEN) != NO_MORE_ELEMENTS:
            yield name.value.decode()
    except:
        drmaa_release_attr_names(attrn_p.contents)
        raise
    else:
        drmaa_release_attr_names(attrn_p.contents)


def vector_attribute_names_iterator():
    attrn_p = pointer(POINTER(drmaa_attr_names_t)())
    c(drmaa_get_vector_attribute_names, attrn_p)
    try:
        name = create_string_buffer(_BUFLEN)
        while drmaa_get_next_attr_name(attrn_p.contents, name,
                                       _BUFLEN) != NO_MORE_ELEMENTS:
            yield name.value.decode()
    except:
        drmaa_release_attr_names(attrn_p.contents)
        raise
    else:
        drmaa_release_attr_names(attrn_p.contents)


def run_bulk_job(jt, start, end, incr=1):
    jids = pointer(POINTER(drmaa_job_ids_t)())
    try:
        c(drmaa_run_bulk_jobs, jids, jt, start, end, incr)
        jid = create_string_buffer(_BUFLEN)
        while drmaa_get_next_job_id(jids.contents, jid,
                                    _BUFLEN) != NO_MORE_ELEMENTS:
            yield jid.value.decode()
    except:
        drmaa_release_job_ids(jids.contents)
        raise
    else:
        drmaa_release_job_ids(jids.contents)


def c(f, *args):
    """
    A helper function wrapping calls to the C DRMAA functions with error
    managing code.
    """
    return f(*(args + (error_buffer, sizeof(error_buffer))))


def string_vector(v):
    vlen = len(v)
    values = (STRING * (vlen + 1))()
    for i, el in enumerate(v):
        if isinstance(el, str):
            el = el.encode(ENCODING)
        elif not isinstance(el, bytes):
            el = str(el).encode(ENCODING)
        values[i] = STRING(el)
    values[vlen] = STRING()
    return values


def attribute_setter(obj, attribute_name):
    """
    returns a drmaa attribute setter
    """
    def f(value):
        "setter for %s" % attribute_name
        c(drmaa_set_attribute, obj, attribute_name, value)
    f.__name__ = 'set_' + attribute_name
    return f


def attribute_getter(obj, attribute_name):
    """
    returns a drmaa attribute setter
    """
    def f():
        "getter for %s" % attribute_name
        attr_buffer = create_string_buffer(1024)
        c(drmaa_get_attribute, obj, attribute_name, attr_buffer,
          sizeof(attr_buffer))
        return attr_buffer.value
    f.__name__ = 'get_' + attribute_name
    return f



# -----------------------------------------------------------
#  Copyright (C) 2008 StatPro Italia s.r.l.
#
#  StatPro Italia
#  Via G. B. Vico 4
#  I-20123 Milano
#  ITALY
#
#  phone: +39 02 96875 1
#  fax:   +39 02 96875 605
#
#  This program is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE. See the license for more details.
# -----------------------------------------------------------
#
#  Author: Enrico Sirola <enrico.sirola@statpro.com>

"""
A python package for DRM job submission and control.

This package is an implementation of the DRMAA 1.0 Python language
binding specification (http://www.ogf.org/documents/GFD.143.pdf).
The source is hosted on GitHub: https://github.com/pygridtools/drmaa-python
Releases are available from PyPi: https://pypi.python.org/pypi/drmaa
Documentation is hosted on ReadTheDocs: http://drmaa-python.readthedocs.org/

:author: Enrico Sirola (enrico.sirola@statpro.com)
:author: Dan Blanchard (dan.blanchard@gmail.com)
"""

from __future__ import absolute_import, print_function, unicode_literals

from .const import (ATTR_BUFFER, BLOCK_EMAIL, CONTACT_BUFFER,
                    control_action_to_string, DEADLINE_TIME, DRM_SYSTEM_BUFFER,
                    DRMAA_IMPLEMENTATION_BUFFER, DURATION_HLIMIT,
                    DURATION_SLIMIT, ERROR_PATH, ERROR_STRING_BUFFER,
                    INPUT_PATH, JOB_CATEGORY, JOB_IDS_SESSION_ALL,
                    JOB_IDS_SESSION_ANY, JOB_NAME, job_state, JobControlAction,
                    JOBNAME_BUFFER, JobState, JobSubmissionState, JOIN_FILES,
                    JS_STATE, NATIVE_SPECIFICATION, NO_MORE_ELEMENTS,
                    OUTPUT_PATH, PLACEHOLDER_HD, PLACEHOLDER_INCR,
                    PLACEHOLDER_WD, REMOTE_COMMAND, SIGNAL_BUFFER, START_TIME,
                    status_to_string, string_to_control_action,
                    submission_state, SUBMISSION_STATE_ACTIVE,
                    SUBMISSION_STATE_HOLD, TIMEOUT_NO_WAIT,
                    TIMEOUT_WAIT_FOREVER, TRANSFER_FILES, V_ARGV, V_EMAIL,
                    V_ENV, WCT_HLIMIT, WCT_SLIMIT, WD)
from .errors import (AlreadyActiveSessionException, AuthorizationException,
                     ConflictingAttributeValuesException,
                     DefaultContactStringException, DeniedByDrmException,
                     DrmCommunicationException, DrmsExitException,
                     DrmsInitException, ExitTimeoutException,
                     HoldInconsistentStateException, IllegalStateException,
                     InternalException, InvalidAttributeFormatException,
                     InvalidContactStringException, InvalidJobException,
                     InvalidJobTemplateException, NoActiveSessionException,
                     NoDefaultContactStringSelectedException,
                     ReleaseInconsistentStateException,
                     ResumeInconsistentStateException,
                     SuspendInconsistentStateException, TryLaterException,
                     UnsupportedAttributeException, InvalidArgumentException,
                     InvalidAttributeValueException, OutOfMemoryException)
from .session import JobInfo, JobTemplate, Session
from .version import __version__, VERSION


__docformat__ = "restructuredtext en"


__all__ = ['JobInfo', 'JobTemplate', 'Session', 'AlreadyActiveSessionException',
           'AuthorizationException', 'ConflictingAttributeValuesException',
           'DefaultContactStringException', 'DeniedByDrmException',
           'DrmCommunicationException', 'DrmsExitException',
           'DrmsInitException', 'ExitTimeoutException',
           'HoldInconsistentStateException', 'IllegalStateException',
           'InternalException', 'InvalidAttributeFormatException',
           'InvalidContactStringException', 'InvalidJobException',
           'InvalidJobTemplateException', 'NoActiveSessionException',
           'NoDefaultContactStringSelectedException',
           'ReleaseInconsistentStateException',
           'ResumeInconsistentStateException',
           'SuspendInconsistentStateException', 'TryLaterException',
           'UnsupportedAttributeException', 'InvalidArgumentException',
           'InvalidAttributeValueException', 'OutOfMemoryException',
           'ATTR_BUFFER', 'BLOCK_EMAIL', 'CONTACT_BUFFER',
           'control_action_to_string', 'DEADLINE_TIME', 'DRM_SYSTEM_BUFFER',
           'DRMAA_IMPLEMENTATION_BUFFER', 'DURATION_HLIMIT', 'DURATION_SLIMIT',
           'ERROR_PATH', 'ERROR_STRING_BUFFER', 'INPUT_PATH', 'JOB_CATEGORY',
           'JOB_IDS_SESSION_ALL', 'JOB_IDS_SESSION_ANY', 'JOB_NAME',
           'job_state', 'JobControlAction', 'JOBNAME_BUFFER', 'JobState',
           'JobSubmissionState', 'JOIN_FILES', 'JS_STATE',
           'NATIVE_SPECIFICATION', 'NO_MORE_ELEMENTS', 'OUTPUT_PATH',
           'PLACEHOLDER_HD', 'PLACEHOLDER_INCR', 'PLACEHOLDER_WD',
           'REMOTE_COMMAND', 'SIGNAL_BUFFER', 'START_TIME', 'status_to_string',
           'string_to_control_action', 'submission_state',
           'SUBMISSION_STATE_ACTIVE', 'SUBMISSION_STATE_HOLD',
           'TIMEOUT_NO_WAIT', 'TIMEOUT_WAIT_FOREVER', 'TRANSFER_FILES',
           'V_ARGV', 'V_EMAIL', 'V_ENV', 'WCT_HLIMIT', 'WCT_SLIMIT', 'WD']



# -----------------------------------------------------------
#  Copyright (C) 2009 StatPro Italia s.r.l.
#
#  StatPro Italia
#  Via G. B. Vico 4
#  I-20123 Milano
#  ITALY
#
#  phone: +39 02 96875 1
#  fax:   +39 02 96875 605
#
#  email: info@riskmap.net
#
#  This program is distributed in the hope that it will be
#  useful, but WITHOUT ANY WARRANTY; without even the
#  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
#  PURPOSE. See the license for more details.
# -----------------------------------------------------------
#
#  Author: Enrico Sirola <enrico.sirola@statpro.com>
#
"""
drmaa constants
"""

from __future__ import absolute_import, print_function, unicode_literals

import locale

# Encoding to use for passing strings to C library
ENCODING = locale.getpreferredencoding()

# drmaa_get_attribute()
ATTR_BUFFER = 1024

# drmaa_get_contact()
CONTACT_BUFFER = 1024

# drmaa_get_DRM_system()
DRM_SYSTEM_BUFFER = 1024

# drmaa_get_DRM_system()
DRMAA_IMPLEMENTATION_BUFFER = 1024

# Agreed buffer length constants
# these are recommended minimum values
ERROR_STRING_BUFFER = 1024
JOBNAME_BUFFER = 1024
SIGNAL_BUFFER = 32

# Agreed constants
TIMEOUT_WAIT_FOREVER = -1
TIMEOUT_NO_WAIT = 0

JOB_IDS_SESSION_ANY = b"DRMAA_JOB_IDS_SESSION_ANY"
JOB_IDS_SESSION_ALL = b"DRMAA_JOB_IDS_SESSION_ALL"

SUBMISSION_STATE_ACTIVE = "drmaa_active"
SUBMISSION_STATE_HOLD = "drmaa_hold"

# Agreed placeholder names
PLACEHOLDER_INCR = b"$drmaa_incr_ph$"
PLACEHOLDER_HD = b"$drmaa_hd_ph$"
PLACEHOLDER_WD = b"$drmaa_wd_ph$"

# Agreed names of job template attributes
REMOTE_COMMAND = b"drmaa_remote_command"
JS_STATE = b"drmaa_js_state"
WD = b"drmaa_wd"
JOB_CATEGORY = b"drmaa_job_category"
NATIVE_SPECIFICATION = b"drmaa_native_specification"
BLOCK_EMAIL = b"drmaa_block_email"
START_TIME = b"drmaa_start_time"
JOB_NAME = b"drmaa_job_name"
INPUT_PATH = b"drmaa_input_path"
OUTPUT_PATH = b"drmaa_output_path"
ERROR_PATH = b"drmaa_error_path"
JOIN_FILES = b"drmaa_join_files"
TRANSFER_FILES = b"drmaa_transfer_files"
DEADLINE_TIME = b"drmaa_deadline_time"
WCT_HLIMIT = b"drmaa_wct_hlimit"
WCT_SLIMIT = b"drmaa_wct_slimit"
DURATION_HLIMIT = b"drmaa_duration_hlimit"
DURATION_SLIMIT = b"drmaa_duration_slimit"

# names of job template vector attributes
V_ARGV = b"drmaa_v_argv"
V_ENV = b"drmaa_v_env"
V_EMAIL = b"drmaa_v_email"

NO_MORE_ELEMENTS = 25


def job_state(code):
    return _JOB_PS[code]


class JobState(object):
    UNDETERMINED = 'undetermined'
    QUEUED_ACTIVE = 'queued_active'
    SYSTEM_ON_HOLD = 'system_on_hold'
    USER_ON_HOLD = 'user_on_hold'
    USER_SYSTEM_ON_HOLD = 'user_system_on_hold'
    RUNNING = 'running'
    SYSTEM_SUSPENDED = 'system_suspended'
    USER_SUSPENDED = 'user_suspended'
    USER_SYSTEM_SUSPENDED = 'user_system_suspended'
    DONE = 'done'
    FAILED = 'failed'

# Job control action


class JobControlAction(object):
    SUSPEND = 'suspend'
    RESUME = 'resume'
    HOLD = 'hold'
    RELEASE = 'release'
    TERMINATE = 'terminate'

_JOB_CONTROL = [JobControlAction.SUSPEND,
                JobControlAction.RESUME,
                JobControlAction.HOLD,
                JobControlAction.RELEASE,
                JobControlAction.TERMINATE]


def string_to_control_action(operation):
    return _JOB_CONTROL.index(operation)


def control_action_to_string(code):
    return _JOB_CONTROL[code]


def status_to_string(status):
    return _JOB_PS[status]

_JOB_PS = {0x00: JobState.UNDETERMINED,
           0x10: JobState.QUEUED_ACTIVE,
           0x11: JobState.SYSTEM_ON_HOLD,
           0x12: JobState.USER_ON_HOLD,
           0x13: JobState.USER_SYSTEM_ON_HOLD,
           0x20: JobState.RUNNING,
           0x21: JobState.SYSTEM_SUSPENDED,
           0x22: JobState.USER_SUSPENDED,
           0x23: JobState.USER_SYSTEM_SUSPENDED,
           0x30: JobState.DONE,
           0x40: JobState.FAILED}

# State at submission time


class JobSubmissionState(object):
    HOLD_STATE = SUBMISSION_STATE_HOLD
    ACTIVE_STATE = SUBMISSION_STATE_ACTIVE

_SUBMISSION_STATE = [JobSubmissionState.HOLD_STATE,
                     JobSubmissionState.ACTIVE_STATE]


def submission_state(code):
    return _SUBMISSION_STATE[code]
